package cn.wisenergy.service.util;

import cn.wisenergy.model.dto.PayPageDto;
import cn.wisenergy.service.wxpay.WxCommon;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.RandomStringUtils;
import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

public class WxPayUtil {

    /***
     * pc端生成统一下单格式的订单，生成一个XML格式的字符串
     */
    public static String createOrderInfo(PayPageDto payPageDto,String tradeNo,String product_id,String time_expire,String appid,String mchid,String key) throws UnsupportedEncodingException {
        String nonce_str = RandomStringUtils.randomAlphanumeric(16);
        SortedMap<String,String> parameters = new TreeMap<>();
        parameters.put("appid", appid);
        parameters.put("mch_id",mchid);
        parameters.put("body", "充值");
        parameters.put("out_trade_no",tradeNo);
        parameters.put("notify_url",WxCommon.NOTIFY_URL);
        DecimalFormat df = new DecimalFormat("#");
        parameters.put("total_fee",  df.format(payPageDto.getTotal()*100));
        parameters.put("nonce_str", nonce_str);
        parameters.put("trade_type","NATIVE");
        parameters.put("product_id", product_id);
        parameters.put("spbill_create_ip","0.0.0.0");
        parameters.put("time_expire",time_expire);
        String sign = createSign(parameters,key);
        parameters.put("sign", sign);//签名
        //将订单对象转为xml格式
        String s = null;
        try {
            return MapToXmlUtils.mapToXml(parameters);//maptoXml方法是微信sdk自带的方法
        } catch (Exception e) {
            e.printStackTrace();
        }
        assert false;
        return new String(s.getBytes(StandardCharsets.UTF_8));
    }


    /***
     * h5生成统一下单格式的订单，生成一个XML格式的字符串
     */
    public static String createOrderInfoH5(PayPageDto payPageDto,String tradeNo, HttpServletRequest request, String appid, String mchid, String key) {
        String nonce_str = RandomStringUtils.randomAlphanumeric(16);
        String sceneInfo="{'h5_info':{'type':'WAP','wap_url': 'https://jygkzy.com','wap_name': '充值'}}";
        String spbill_create_ip = getRealIp(request);
        SortedMap<String,String> parameters = new TreeMap<>();
        parameters.put("appid", appid);
        parameters.put("mch_id", mchid);
        parameters.put("body", "充值");
        parameters.put("out_trade_no", tradeNo);
        parameters.put("nonce_str", nonce_str);
        DecimalFormat df = new DecimalFormat("#");
        parameters.put("total_fee",  df.format(payPageDto.getTotal()*100));
        parameters.put("notify_url",WxCommon.NOTIFY_URL);
        parameters.put("trade_type","MWEB");
        parameters.put("scene_info",sceneInfo);
        parameters.put("spbill_create_ip",spbill_create_ip);
        String sign = createSign(parameters,key);
        parameters.put("sign", sign);//签名
        //将订单对象转为xml格式
        String s = null;
        try {
            return MapToXmlUtils.mapToXml(parameters);//maptoXml方法是微信sdk自带的方法
        } catch (Exception e) {
            e.printStackTrace();
        }
        assert false;
        return new String(s.getBytes(StandardCharsets.UTF_8));
    }


    /***
     * 小程序生成统一下单格式的订单，生成一个XML格式的字符串
     */
    public static String createOrderInfoWx(PayPageDto payPageDto,String tradeNo,String nonce_str,String Applets_ID,String secrt_key,String mchid,String key) {
        Map<String, Object> infoByCode = WxUtil.getInfoByCode(payPageDto.getCode(), Applets_ID, secrt_key);
        SortedMap<String,String> parameters = new TreeMap<>();
        parameters.put("appid", Applets_ID);
        parameters.put("mch_id", mchid);
        parameters.put("body", "充值");
        parameters.put("out_trade_no", tradeNo);
        parameters.put("notify_url",WxCommon.NOTIFY_URL);
        parameters.put("nonce_str", nonce_str);
        DecimalFormat df = new DecimalFormat("#");
        parameters.put("total_fee",  df.format(payPageDto.getTotal()*100));
        parameters.put("trade_type","JSAPI");
        parameters.put("openid", infoByCode.get("openid").toString());
        parameters.put("spbill_create_ip","0.0.0.0");
        String sign = createSign(parameters,key);
        parameters.put("sign", sign);//签名
        //将订单对象转为xml格式
        String s = null;

        try {
            return MapToXmlUtils.mapToXml(parameters);//maptoXml方法是微信sdk自带的方法
        } catch (Exception e) {
            e.printStackTrace();
        }
        assert false;
        return new String(s.getBytes(StandardCharsets.UTF_8));
    }

    /***
     * 生成签名
     */
    public static String createSign(SortedMap<String,String> parameters,String key)  {
        // 多线程访问的情况下需要用StringBuffer
        StringBuilder sb = new StringBuilder();
        // 所有参与传参的key按照accsii排序（升序）
        Set es = parameters.keySet();
        for (Object set : es) {
            String k = set.toString();
            Object v = parameters.get(k);
            sb.append(k)
                    .append("=")
                    .append(v.toString())
                    .append("&");
        }
        sb.append("key=")
                .append(key);
        return DigestUtils.md5Hex(sb.toString()).toUpperCase();

    }

    /**
     * 获取真实ip地址 通过阿帕奇代理的也能获取到真实ip
     */
    public static String getRealIp(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

    /**
     * 设置微信二维码失效时间，并返回具体失效的时间点
     */
    public static String getOrderExpireTime(Long expire){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        Date now = new Date();
        Date afterDate = new Date(now .getTime() + expire);
        return sdf.format(afterDate );
    }

    /**
     * 生成16位唯一订单号
     * @return
     */
    public static String getOrderIdByUUId() {
        int first = new Random(10).nextInt(8) + 1;
        int hashCodeV = UUID.randomUUID().toString().hashCode();
        if (hashCodeV < 0) {//有可能是负数
            hashCodeV = -hashCodeV;
        }
        return first + String.format("%015d", hashCodeV);
    }
}
